package jump.basics;

/**

	JumpSet is a generic Container that can be used as a typesafe vector, list or stack.

	When creating a new container you can provide an integer as default argument which
	is the delta used to increase the size when the Container needs to grow.

	New elements can be appended as last element using add or push.
	Alternatively they can be inserted at any position using insAt.

	Vice versa elements can be popped from the end or removed at a certain position.

	To search for an element use find. If the type of the container is native the first
	equal element will be found, if it is a reference type the same element will be found.

	To sort the set use sort. Native types are sorted comparing their values.
	By default reference types are sorted comparing their haschCode() values. This can be 
	overridden by deriving a new class from BinPredicate with providing a own implemantation of
	public int exec( Object o1, Object o2 );

	Copyright ( c ) 1996 Detlef Hoeffner

*/

template < class T > public class JumpSet
{
protected:
	int delta, cur, max;
	T data[];
	void Grow( int newmax ) {
		T temp[] = new T[ max = newmax ];
		if( cur > 0 ) System.arraycopy( data, 0, temp, 0, cur );
		data = temp; }
public:
	JumpSet( int d = 100 ) { delta = d; }
	JumpSet( T[] array, int d = 100 ) { data = array; cur = max = array.length; delta = d; }
	JumpSet( JumpSet s ) {
		data = new T[ max = s.max ];
		delta = s.delta;
		System.arraycopy( s.data, 0, data, 0, cur ); }
	Object clone() { return new JumpSet( this ); }
	boolean equals( Object obj ) { return obj instanceof JumpSet && equals( (JumpSet)obj ); }
	boolean equals( JumpSet s ) {
		if( s.delta == delta && s.cur == cur && s.max == max ) {
			for( int i = cur; i-- > 0; ) if( data[i] != s.data[i] ) return false;
			return true; }
		return false; }
	void clear() { cur = max = 0; data = null; }
	final int max() { return max; }
	final int size() { return cur; }
	final T[] getData() { return data; }
	final T getAt( int at ) { return data[ at ]; }
	final T operator[]( int at ) { return data[ at ]; }
	int add( T t ) { if( cur >= max ) Grow( max + delta ); data[ cur ] = t; return cur++; }
	final void push( T t ) { if( cur >= max ) Grow( max + delta ); data[ cur++ ] = t; }
	final T pop() { if( cur <= 0 ) throw new ArrayIndexOutOfBoundsException( cur ); return data[ --cur ]; }
	void insAt( int at, T t ) {
		if( at >= max ) Grow( at + delta );
		else if( cur >= max ) Grow( max + delta );
		if( at >= cur ) cur = at + 1;
		else System.arraycopy( data, at, data, at + 1, cur++ - at );
		data[ at ] = t; }
	void setAt( int at, T t ) { if( at >= max ) Grow( at + delta ); data[ at ] = t; }
	final boolean rmv( T t ) { return rmvAt( find( t ) ); }
	boolean rmvAt( int at ) {
		if( at < 0 || at >= cur ) return false;
		System.arraycopy( data, at + 1, data, at, --cur - at );
		return true; }
	int find( T t ) {
		for( int i = cur; i-- > 0; ) if( data[ i ] == t ) return i;
		return -1; }
	int forEach( UnPredicate perform, int startAt = 0, int endBefore = -1 ) {
		if( endBefore < 0 || endBefore > cur ) endBefore = cur;
		int i = startAt;
		while( i < endBefore ) perform.exec( data[ i++ ] );
		return i - startAt; }
	void sort( BinPredicate comp ) { Sort.quickSort( data, 0, cur - 1, comp ); }
}
